@@ -153,8 +153,13 @@ module Make (Ast : Sig.Camlp4Ast) = struct
   value array_function loc str name = with_loc (array_function_no_loc str name) loc;
   value mkrf =
     fun
-    [ <:rec_flag< rec >> -> Recursive
-    | <:rec_flag<>> -> Nonrecursive
+    [ Ast.ReRecursive -> Recursive
+    | Ast.ReNonrecursive | Ast.ReNil -> Nonrecursive
+    | _ -> assert False ];
+  value mknrf =
+    fun
+    [ Ast.ReNonrecursive -> Nonrecursive
+    | Ast.ReRecursive | Ast.ReNil -> Recursive
     | _ -> assert False ];
 
   value mkli sloc s list = with_loc (loop lident list) sloc
@@ -174,9 +179,7 @@ module Make (Ast : Sig.Camlp4Ast) = struct
 
     let rec self i acc =
       match i with
-      [ <:ident< $lid:"*predef*"$.$lid:"option"$ >> ->
-          (ldot (lident "*predef*") "option", `lident)
-      | <:ident< $i1$.$i2$ >> ->
+      [ <:ident< $i1$.$i2$ >> ->
           self i2 (Some (self i1 acc))
       | <:ident< $i1$ $i2$ >> ->
           let i' = Lapply (fst (self i1 None)) (fst (self i2 None)) in
@@ -249,12 +252,9 @@ module Make (Ast : Sig.Camlp4Ast) = struct
   value rec ty_var_list_of_ctyp =
     fun
     [ <:ctyp< $t1$ $t2$ >> -> ty_var_list_of_ctyp t1 @ ty_var_list_of_ctyp t2
-    | <:ctyp< '$s$ >> -> [s]
+    | <:ctyp@loc< '$s$ >> -> [with_loc s loc]
     | _ -> assert False ];
 
-  value predef_option loc =
-    TyId (loc, IdAcc (loc, IdLid (loc, "*predef*"), IdLid (loc, "option")));
-
   value attribute_fwd = ref (fun _ _ _ -> assert False);
 
   value attribute loc s str =
@@ -280,11 +280,10 @@ module Make (Ast : Sig.Camlp4Ast) = struct
         if is_cls then mktyp loc (Ptyp_class li (List.map ctyp al))
         else mktyp loc (Ptyp_constr li (List.map ctyp al))
     | TyArr loc (TyLab _ lab t1) t2 ->
-        mktyp loc (Ptyp_arrow lab (ctyp t1) (ctyp t2))
-    | TyArr loc (TyOlb loc1 lab t1) t2 ->
-        let t1 = TyApp loc1 (predef_option loc1) t1 in
-        mktyp loc (Ptyp_arrow ("?" ^ lab) (ctyp t1) (ctyp t2))
-    | TyArr loc t1 t2 -> mktyp loc (Ptyp_arrow "" (ctyp t1) (ctyp t2))
+        mktyp loc (Ptyp_arrow (Labelled lab) (ctyp t1) (ctyp t2))
+    | TyArr loc (TyOlb _ lab t1) t2 ->
+        mktyp loc (Ptyp_arrow (Optional lab) (ctyp t1) (ctyp t2))
+    | TyArr loc t1 t2 -> mktyp loc (Ptyp_arrow Nolabel (ctyp t1) (ctyp t2))
     | <:ctyp@loc< < $fl$ > >> -> mktyp loc (Ptyp_object (meth_list fl []) Closed)
     | <:ctyp@loc< < $fl$ .. > >> ->
         mktyp loc (Ptyp_object (meth_list fl []) Open)
@@ -349,7 +348,7 @@ module Make (Ast : Sig.Camlp4Ast) = struct
     match fl with
     [ <:ctyp<>> -> acc
     | <:ctyp< $t1$; $t2$ >> -> meth_list t1 (meth_list t2 acc)
-    | <:ctyp< $lid:lab$ : $t$ >> -> [(lab, [], mkpolytype (ctyp t)) :: acc]
+    | <:ctyp@loc< $lid:lab$ : $t$ >> -> [(with_loc lab loc, [], mkpolytype (ctyp t)) :: acc]
     | _ -> assert False ]
 
   and package_type_constraints wc acc =
@@ -407,34 +406,54 @@ module Make (Ast : Sig.Camlp4Ast) = struct
   value mkvariant =
     fun
     [ <:ctyp@loc< $id:(<:ident@sloc< $uid:s$ >>)$ >> ->
-      {pcd_name = with_loc (conv_con s) sloc; pcd_args = []; pcd_res = None; pcd_loc = mkloc loc; pcd_attributes = []}
+      { pcd_name = with_loc (conv_con s) sloc
+      ; pcd_args = Pcstr_tuple []
+      ; pcd_res = None
+      ; pcd_loc = mkloc loc
+      ; pcd_attributes = []
+      }
     | <:ctyp@loc< $id:(<:ident@sloc< $uid:s$ >>)$ of $t$ >> ->
-        {pcd_name = with_loc (conv_con s) sloc; pcd_args = List.map ctyp (list_of_ctyp t []); pcd_res = None; pcd_loc = mkloc loc; pcd_attributes = []}
+        { pcd_name = with_loc (conv_con s) sloc
+        ; pcd_args = Pcstr_tuple (List.map ctyp (list_of_ctyp t []))
+        ; pcd_res = None
+        ; pcd_loc = mkloc loc
+        ; pcd_attributes = []
+        }
     | <:ctyp@loc< $id:(<:ident@sloc< $uid:s$ >>)$ : ($t$ -> $u$) >> ->
-        {pcd_name = with_loc (conv_con s) sloc; pcd_args = List.map ctyp (list_of_ctyp t []); pcd_res = Some (ctyp u); pcd_loc = mkloc loc; pcd_attributes = []}
+        { pcd_name = with_loc (conv_con s) sloc
+        ; pcd_args = Pcstr_tuple (List.map ctyp (list_of_ctyp t []))
+        ; pcd_res = Some (ctyp u)
+        ; pcd_loc = mkloc loc
+        ; pcd_attributes = []
+        }
     | <:ctyp@loc< $id:(<:ident@sloc< $uid:s$ >>)$ : $t$ >> ->
-        {pcd_name = with_loc (conv_con s) sloc; pcd_args = []; pcd_res = Some (ctyp t); pcd_loc = mkloc loc; pcd_attributes = []}
+        { pcd_name = with_loc (conv_con s) sloc
+        ; pcd_args = Pcstr_tuple []
+        ; pcd_res = Some (ctyp t)
+        ; pcd_loc = mkloc loc
+        ; pcd_attributes = []
+        }
     | _ -> assert False (*FIXME*) ];
   value mkextension_constructor =
     fun
     [ <:ctyp@loc< $id:(<:ident@sloc< $uid:s$ >>)$ >> ->
        {pext_name = with_loc (conv_con s) sloc;
-        pext_kind = Pext_decl [] None;
+        pext_kind = Pext_decl (Pcstr_tuple []) None;
         pext_loc  = mkloc loc;
         pext_attributes = []}
     | <:ctyp@loc< $id:(<:ident@sloc< $uid:s$ >>)$ of $t$ >> ->
        {pext_name = with_loc (conv_con s) sloc;
-        pext_kind = Pext_decl (List.map ctyp (list_of_ctyp t [])) None;
+        pext_kind = Pext_decl (Pcstr_tuple (List.map ctyp (list_of_ctyp t []))) None;
         pext_loc  = mkloc loc;
         pext_attributes = []}
     | <:ctyp@loc< $id:(<:ident@sloc< $uid:s$ >>)$ : ($t$ -> $u$) >> ->
        {pext_name = with_loc (conv_con s) sloc;
-        pext_kind = Pext_decl (List.map ctyp (list_of_ctyp t [])) (Some (ctyp u));
+        pext_kind = Pext_decl (Pcstr_tuple (List.map ctyp (list_of_ctyp t []))) (Some (ctyp u));
         pext_loc  = mkloc loc;
         pext_attributes = []}
     | <:ctyp@loc< $id:(<:ident@sloc< $uid:s$ >>)$ : $t$ >> ->
        {pext_name = with_loc (conv_con s) sloc;
-        pext_kind = Pext_decl [] (Some (ctyp t));
+        pext_kind = Pext_decl (Pcstr_tuple []) (Some (ctyp t));
         pext_loc  = mkloc loc;
         pext_attributes = []}
     | <:ctyp@loc< $id:(<:ident@sloc< $uid:s$ >>)$ == $id:r$  >> ->
@@ -609,19 +628,19 @@ module Make (Ast : Sig.Camlp4Ast) = struct
   ;
 
   value rec deep_mkrangepat loc c1 c2 =
-    if c1 = c2 then mkghpat loc (Ppat_constant (Const_char c1))
+    if c1 = c2 then mkghpat loc (Ppat_constant (Pconst_char c1))
     else
       mkghpat loc
-        (Ppat_or (mkghpat loc (Ppat_constant (Const_char c1)))
+        (Ppat_or (mkghpat loc (Ppat_constant (Pconst_char c1)))
           (deep_mkrangepat loc (Char.chr (Char.code c1 + 1)) c2))
   ;
 
   value rec mkrangepat loc c1 c2 =
     if c1 > c2 then mkrangepat loc c2 c1
-    else if c1 = c2 then mkpat loc (Ppat_constant (Const_char c1))
+    else if c1 = c2 then mkpat loc (Ppat_constant (Pconst_char c1))
     else
       mkpat loc
-        (Ppat_or (mkghpat loc (Ppat_constant (Const_char c1)))
+        (Ppat_or (mkghpat loc (Ppat_constant (Pconst_char c1)))
           (deep_mkrangepat loc (Char.chr (Char.code c1 + 1)) c2))
   ;
 
@@ -667,24 +686,12 @@ module Make (Ast : Sig.Camlp4Ast) = struct
               "this is not a constructor, it cannot be applied in a pattern" ]
     | PaArr loc p -> mkpat loc (Ppat_array (List.map patt (list_of_patt p [])))
     | PaChr loc s ->
-        mkpat loc (Ppat_constant (Const_char (char_of_char_token loc s)))
-    | PaInt loc s ->
-        let i = try int_of_string s with [
-          Failure _ -> error loc "Integer literal exceeds the range of representable integers of type int"
-        ] in mkpat loc (Ppat_constant (Const_int i))
-    | PaInt32 loc s ->
-        let i32 = try Int32.of_string s with [
-          Failure _ -> error loc "Integer literal exceeds the range of representable integers of type int32"
-        ] in mkpat loc (Ppat_constant (Const_int32 i32))
-    | PaInt64 loc s ->
-        let i64 = try Int64.of_string s with [
-          Failure _ -> error loc "Integer literal exceeds the range of representable integers of type int64"
-        ] in mkpat loc (Ppat_constant (Const_int64 i64))
-    | PaNativeInt loc s ->
-        let nati = try Nativeint.of_string s with [
-          Failure _ -> error loc "Integer literal exceeds the range of representable integers of type nativeint"
-        ] in mkpat loc (Ppat_constant (Const_nativeint nati))
-    | PaFlo loc s -> mkpat loc (Ppat_constant (Const_float (remove_underscores s)))
+        mkpat loc (Ppat_constant (Pconst_char (char_of_char_token loc s)))
+    | PaInt loc s ->   mkpat loc (Ppat_constant (Pconst_integer (s, None)))
+    | PaInt32 loc s -> mkpat loc (Ppat_constant (Pconst_integer (s, Some 'l')))
+    | PaInt64 loc s -> mkpat loc (Ppat_constant (Pconst_integer (s, Some 'L')))
+    | PaNativeInt loc s -> mkpat loc (Ppat_constant (Pconst_integer (s, Some 'n')))
+    | PaFlo loc s -> mkpat loc (Ppat_constant (Pconst_float (remove_underscores s, None)))
     | PaLab loc _ _ -> error loc "labeled pattern not allowed here"
     | PaOlb loc _ _ | PaOlbi loc _ _ _ -> error loc "labeled pattern not allowed here"
     | PaOrp loc p1 p2 -> mkpat loc (Ppat_or (patt p1) (patt p2))
@@ -702,7 +709,7 @@ module Make (Ast : Sig.Camlp4Ast) = struct
         let is_closed = if wildcards = [] then Closed else Open in
         mkpat loc (Ppat_record (List.map mklabpat ps, is_closed))
     | PaStr loc s ->
-        mkpat loc (Ppat_constant (Const_string (string_of_string_token loc s) None))
+        mkpat loc (Ppat_constant (Pconst_string (string_of_string_token loc s) None))
     | <:patt@loc< ($p1$, $p2$) >> ->
          mkpat loc (Ppat_tuple
            (List.map patt (list_of_patt p1 (list_of_patt p2 []))))
@@ -777,7 +784,8 @@ value varify_constructors var_names =
       | Ptyp_arrow label core_type core_type' ->
           Ptyp_arrow label (loop core_type) (loop core_type')
       | Ptyp_tuple lst -> Ptyp_tuple (List.map loop lst)
-      | Ptyp_constr ({ txt = Lident s }) [] when List.mem s var_names ->
+      | Ptyp_constr ({ txt = Lident s }) []
+          when List.exists (fun x -> s = x.txt) var_names ->
           Ptyp_var ("&" ^ s)
       | Ptyp_constr longident lst ->
           Ptyp_constr longident (List.map loop lst)
@@ -813,7 +821,8 @@ value varify_constructors var_names =
     fun
     [ <:expr@loc< $x$.val >> ->
         mkexp loc
-          (Pexp_apply (mkexp loc (Pexp_ident (lident_with_loc "!" loc))) [("", expr x)])
+          (Pexp_apply (mkexp loc (Pexp_ident (lident_with_loc "!" loc)))
+             [(Nolabel, expr x)])
     | ExAcc loc _ _ | <:expr@loc< $id:<:ident< $_$ . $_$ >>$ >> as e ->
         let (e, l) =
           match sep_expr_acc [] e with
@@ -859,7 +868,7 @@ value varify_constructors var_names =
     | ExAre loc e1 e2 ->
         mkexp loc
           (Pexp_apply (mkexp loc (Pexp_ident (array_function loc "Array" "get")))
-            [("", expr e1); ("", expr e2)])
+            [(Nolabel, expr e1); (Nolabel, expr e2)])
     | ExArr loc e -> mkexp loc (Pexp_array (List.map expr (list_of_expr e [])))
     | ExAsf loc ->
         mkexp loc (Pexp_assert (mkexp loc (Pexp_construct {txt=Lident "false"; loc=mkloc loc} None)))
@@ -868,66 +877,57 @@ value varify_constructors var_names =
           match e with
           [ <:expr@loc< $x$.val >> ->
               Pexp_apply (mkexp loc (Pexp_ident (lident_with_loc ":=" loc)))
-                [("", expr x); ("", expr v)]
+                [(Nolabel, expr x); (Nolabel, expr v)]
           | ExAcc loc _ _ ->
               match (expr e).pexp_desc with
               [ Pexp_field e lab -> Pexp_setfield e lab (expr v)
               | _ -> error loc "bad record access" ]
           | ExAre loc e1 e2 ->
               Pexp_apply (mkexp loc (Pexp_ident (array_function loc "Array" "set")))
-                [("", expr e1); ("", expr e2); ("", expr v)]
+                [(Nolabel, expr e1); (Nolabel, expr e2); (Nolabel, expr v)]
           | <:expr< $id:(<:ident@lloc< $lid:lab$ >>)$ >> -> Pexp_setinstvar (with_loc lab lloc) (expr v)
           | ExSte loc e1 e2 ->
               Pexp_apply
                 (mkexp loc (Pexp_ident (array_function loc "String" "set")))
-                [("", expr e1); ("", expr e2); ("", expr v)]
+                [(Nolabel, expr e1); (Nolabel, expr e2); (Nolabel, expr v)]
           | _ -> error loc "bad left part of assignment" ]
         in
         mkexp loc e
     | ExAsr loc e -> mkexp loc (Pexp_assert (expr e))
     | ExChr loc s ->
-        mkexp loc (Pexp_constant (Const_char (char_of_char_token loc s)))
+        mkexp loc (Pexp_constant (Pconst_char (char_of_char_token loc s)))
     | ExCoe loc e t1 t2 ->
         let t1 =
           match t1 with
           [ <:ctyp<>> -> None
           | t -> Some (ctyp t) ] in
         mkexp loc (Pexp_coerce (expr e) t1 (ctyp t2))
-    | ExFlo loc s -> mkexp loc (Pexp_constant (Const_float (remove_underscores s)))
+    | ExFlo loc s -> mkexp loc (Pexp_constant (Pconst_float (remove_underscores s, None)))
     | ExFor loc p e1 e2 df el ->
         let e3 = ExSeq loc el in
         mkexp loc (Pexp_for (patt p) (expr e1) (expr e2) (mkdirection df) (expr e3))
     | <:expr@loc< fun [ $PaLab _ lab po$ when $w$ -> $e$ ] >> ->
-        mkfun loc lab None (patt_of_lab loc lab po) e w
+        mkfun loc (Labelled lab) None (patt_of_lab loc lab po) e w
     | <:expr@loc< fun [ $PaOlbi _ lab p e1$ when $w$ -> $e2$ ] >> ->
         let lab = paolab lab p in
-        mkfun loc ("?" ^ lab) (Some (expr e1)) (patt p) e2 w
+        mkfun loc (Optional lab) (Some (expr e1)) (patt p) e2 w
     | <:expr@loc< fun [ $PaOlb _ lab p$ when $w$ -> $e$ ] >> ->
         let lab = paolab lab p in
-        mkfun loc ("?" ^ lab) None (patt_of_lab loc lab p) e w
+        mkfun loc (Optional lab) None (patt_of_lab loc lab p) e w
     | ExFun loc a -> mkexp loc (Pexp_function (match_case a []))
     | ExIfe loc e1 e2 e3 ->
         mkexp loc (Pexp_ifthenelse (expr e1) (expr e2) (Some (expr e3)))
-    | ExInt loc s ->
-        let i = try int_of_string s with [
-          Failure _ -> error loc "Integer literal exceeds the range of representable integers of type int"
-        ] in mkexp loc (Pexp_constant (Const_int i))
-    | ExInt32 loc s ->
-        let i32 = try Int32.of_string s with [
-          Failure _ -> error loc "Integer literal exceeds the range of representable integers of type int32"
-        ] in mkexp loc (Pexp_constant (Const_int32 i32))
-    | ExInt64 loc s ->
-        let i64 = try Int64.of_string s with [
-          Failure _ -> error loc "Integer literal exceeds the range of representable integers of type int64"
-        ] in mkexp loc (Pexp_constant (Const_int64 i64))
-    | ExNativeInt loc s ->
-        let nati = try Nativeint.of_string s with [
-          Failure _ -> error loc "Integer literal exceeds the range of representable integers of type nativeint"
-        ] in mkexp loc (Pexp_constant (Const_nativeint nati))
+    | ExInt loc s ->   mkexp loc (Pexp_constant (Pconst_integer (s, None)))
+    | ExInt32 loc s -> mkexp loc (Pexp_constant (Pconst_integer (s, Some 'l')))
+    | ExInt64 loc s -> mkexp loc (Pexp_constant (Pconst_integer (s, Some 'L')))
+    | ExNativeInt loc s -> mkexp loc (Pexp_constant (Pconst_integer (s, Some 'n')))
     | ExLab loc _ _ -> error loc "labeled expression not allowed here"
     | ExLaz loc e -> mkexp loc (Pexp_lazy (expr e))
     | ExLet loc rf bi e ->
-        mkexp loc (Pexp_let (mkrf rf) (binding bi []) (expr e))
+        let e = expr e in
+        match binding bi [] with
+        [ [] -> e
+        | bi -> mkexp loc (Pexp_let (mkrf rf) bi e) ]
     | ExLmd loc i me e -> mkexp loc (Pexp_letmodule (with_loc i loc) (module_expr me) (expr e))
     | ExMat loc e a -> mkexp loc (Pexp_match (expr e) (match_case a []))
     | ExNew loc id -> mkexp loc (Pexp_new (long_type_ident id))
@@ -960,13 +960,13 @@ value varify_constructors var_names =
               mkexp _loc (Pexp_sequence (expr e) (loop el)) ]
         in
         loop (list_of_expr e [])
-    | ExSnd loc e s -> mkexp loc (Pexp_send (expr e) s)
+    | ExSnd loc e s -> mkexp loc (Pexp_send (expr e) (with_loc s loc))
     | ExSte loc e1 e2 ->
         mkexp loc
           (Pexp_apply (mkexp loc (Pexp_ident (array_function loc "String" "get")))
-            [("", expr e1); ("", expr e2)])
+            [(Nolabel, expr e1); (Nolabel, expr e2)])
     | ExStr loc s ->
-        mkexp loc (Pexp_constant (Const_string (string_of_string_token loc s) None))
+        mkexp loc (Pexp_constant (Pconst_string (string_of_string_token loc s) None))
     | ExTry loc e a -> mkexp loc (Pexp_try (expr e) (match_case a []))
     | <:expr@loc< ($e1$, $e2$) >> ->
          mkexp loc (Pexp_tuple (List.map expr (list_of_expr e1 (list_of_expr e2 []))))
@@ -991,7 +991,7 @@ value varify_constructors var_names =
     | <:expr@loc< (module $me$) >> ->
         mkexp loc (Pexp_pack (module_expr me))
     | ExFUN loc i e ->
-        mkexp loc (Pexp_newtype i (expr e))
+        mkexp loc (Pexp_newtype (with_loc i loc) (expr e))
     | <:expr@loc< $_$,$_$ >> -> error loc "expr, expr: not allowed here"
     | <:expr@loc< $_$;$_$ >> ->
         error loc "expr; expr: not allowed here, use do {...} or [|...|] to surround them"
@@ -1009,9 +1009,9 @@ value varify_constructors var_names =
     | e -> expr e ]
   and label_expr =
     fun
-    [ ExLab loc lab eo -> (lab, expr_of_lab loc lab eo)
-    | ExOlb loc lab eo -> ("?" ^ lab, expr_of_lab loc lab eo)
-    | e -> ("", expr e) ]
+    [ ExLab loc lab eo -> (Labelled lab, expr_of_lab loc lab eo)
+    | ExOlb loc lab eo -> (Optional lab, expr_of_lab loc lab eo)
+    | e -> (Nolabel, expr e) ]
   and binding x acc =
     match x with
     [ <:binding< $x$ and $y$ >> ->
@@ -1020,12 +1020,13 @@ value varify_constructors var_names =
       (* this code is not pretty because it is temporary *)
       let rec id_to_string x =
         match x with
-            [ <:ctyp< $lid:x$ >> -> [x]
+            [ <:ctyp@loc< $lid:x$ >> -> [with_loc x loc]
             | <:ctyp< $x$ $y$ >> -> (id_to_string x) @ (id_to_string y)
             | _ -> assert False]
       in
       let vars = id_to_string vs in
-      let ampersand_vars = List.map (fun x -> "&" ^ x) vars in
+      let ampersand_vars = List.map (fun x ->
+        { loc = x.loc; txt = "&" ^ x.txt}) vars in
       let ty' = varify_constructors vars (ctyp ty) in
       let mkexp = mkexp _loc in
       let mkpat = mkpat _loc in
@@ -1150,13 +1151,13 @@ value varify_constructors var_names =
     | SgDir _ _ _ -> l
     | <:sig_item@loc< exception $uid:s$ >> ->
         [mksig loc (Psig_exception { pext_name       = with_loc (conv_con s) loc
-                                   ; pext_kind       = Pext_decl ([], None)
+                                   ; pext_kind       = Pext_decl (Pcstr_tuple [], None)
                                    ; pext_attributes = []
                                    ; pext_loc        = mkloc loc })
          :: l]
     | <:sig_item@loc< exception $uid:s$ of $t$ >> ->
         [mksig loc (Psig_exception { pext_name       = with_loc (conv_con s) loc
-                                   ; pext_kind       = Pext_decl (List.map ctyp (list_of_ctyp t []), None)
+                                   ; pext_kind       = Pext_decl (Pcstr_tuple (List.map ctyp (list_of_ctyp t [])), None)
                                    ; pext_attributes = []
                                    ; pext_loc        = mkloc loc })
          :: l]
@@ -1179,11 +1180,12 @@ value varify_constructors var_names =
         let fresh = override_flag loc ov in
         [mksig loc (Psig_open {popen_override=fresh; popen_lid=long_uident id;
                                popen_attributes=[]; popen_loc = mkloc loc}) :: l]
-    | SgTyp loc tdl ->
+    | SgTyp loc rf tdl ->
+      let rf = mknrf rf in
       let ty =
         match mktype_decl_or_ext tdl `Unknown with
-        [ `Unknown -> Psig_type []
-        | `Dcl l -> Psig_type l
+        [ `Unknown -> Psig_type (rf, [])
+        | `Dcl l -> Psig_type (rf, l)
         | `Ext e -> Psig_typext e ]
       in
       [mksig loc ty :: l]
@@ -1247,13 +1249,13 @@ value varify_constructors var_names =
     | StDir _ _ _ -> l
     | <:str_item@loc< exception $uid:s$ >> ->
         [mkstr loc (Pstr_exception { pext_name       = with_loc (conv_con s) loc
-                                   ; pext_kind       = Pext_decl ([], None)
+                                   ; pext_kind       = Pext_decl (Pcstr_tuple [], None)
                                    ; pext_attributes = []
                                    ; pext_loc        = mkloc loc })
          :: l ]
     | <:str_item@loc< exception $uid:s$ of $t$ >> ->
         [mkstr loc (Pstr_exception { pext_name       = with_loc (conv_con s) loc
-                                   ; pext_kind       = Pext_decl (List.map ctyp (list_of_ctyp t []), None)
+                                   ; pext_kind       = Pext_decl (Pcstr_tuple (List.map ctyp (list_of_ctyp t [])), None)
                                    ; pext_attributes = []
                                    ; pext_loc        = mkloc loc })
          :: l ]
@@ -1287,11 +1289,12 @@ value varify_constructors var_names =
                                popen_lid=long_uident id;
                                popen_attributes=[];
                                popen_loc=mkloc loc}) :: l]
-    | StTyp loc tdl ->
+    | StTyp loc rf tdl ->
+      let rf = mknrf rf in
       let ty =
         match mktype_decl_or_ext tdl `Unknown with
-        [ `Unknown -> Pstr_type []
-        | `Dcl l -> Pstr_type l
+        [ `Unknown -> Pstr_type (rf, [])
+        | `Dcl l -> Pstr_type (rf, l)
         | `Ext e -> Pstr_typext e ]
       in
       [mkstr loc ty :: l]
@@ -1304,11 +1307,10 @@ value varify_constructors var_names =
         mkcty loc
           (Pcty_constr (long_class_ident id) (List.map ctyp (list_of_opt_ctyp tl [])))
     | CtFun loc (TyLab _ lab t) ct ->
-        mkcty loc (Pcty_arrow lab (ctyp t) (class_type ct))
-    | CtFun loc (TyOlb loc1 lab t) ct ->
-        let t = TyApp loc1 (predef_option loc1) t in
-        mkcty loc (Pcty_arrow ("?" ^ lab) (ctyp t) (class_type ct))
-    | CtFun loc t ct -> mkcty loc (Pcty_arrow "" (ctyp t) (class_type ct))
+        mkcty loc (Pcty_arrow (Labelled lab) (ctyp t) (class_type ct))
+    | CtFun loc (TyOlb _ lab t) ct ->
+        mkcty loc (Pcty_arrow (Optional lab) (ctyp t) (class_type ct))
+    | CtFun loc t ct -> mkcty loc (Pcty_arrow Nolabel (ctyp t) (class_type ct))
     | CtSig loc t_o ctfl ->
         let t =
           match t_o with
@@ -1370,11 +1372,11 @@ value varify_constructors var_names =
         class_sig_item csg1 (class_sig_item csg2 l)
     | CgInh loc ct -> [mkctf loc (Pctf_inherit (class_type ct)) :: l]
     | CgMth loc s pf t ->
-        [mkctf loc (Pctf_method (s, mkprivate pf, Concrete, mkpolytype (ctyp t))) :: l]
+        [mkctf loc (Pctf_method (with_loc s loc, mkprivate pf, Concrete, mkpolytype (ctyp t))) :: l]
     | CgVal loc s b v t ->
-        [mkctf loc (Pctf_val (s, mkmutable b, mkvirtual v, ctyp t)) :: l]
+        [mkctf loc (Pctf_val (with_loc s loc, mkmutable b, mkvirtual v, ctyp t)) :: l]
     | CgVir loc s b t ->
-        [mkctf loc (Pctf_method (s, mkprivate b, Virtual, mkpolytype (ctyp t))) :: l]
+        [mkctf loc (Pctf_method (with_loc s loc, mkprivate b, Virtual, mkpolytype (ctyp t))) :: l]
     | CgAnt _ _ -> assert False ]
   and class_expr =
     fun
@@ -1387,15 +1389,15 @@ value varify_constructors var_names =
           (Pcl_constr (long_class_ident id) (List.map ctyp (list_of_opt_ctyp tl [])))
     | CeFun loc (PaLab _ lab po) ce ->
         mkcl loc
-          (Pcl_fun lab None (patt_of_lab loc lab po) (class_expr ce))
+          (Pcl_fun (Labelled lab) None (patt_of_lab loc lab po) (class_expr ce))
     | CeFun loc (PaOlbi _ lab p e) ce ->
         let lab = paolab lab p in
-        mkcl loc (Pcl_fun ("?" ^ lab) (Some (expr e)) (patt p) (class_expr ce))
+        mkcl loc (Pcl_fun (Optional lab) (Some (expr e)) (patt p) (class_expr ce))
     | CeFun loc (PaOlb _ lab p) ce ->
         let lab = paolab lab p in
         mkcl loc
-          (Pcl_fun ("?" ^ lab) None (patt_of_lab loc lab p) (class_expr ce))
-    | CeFun loc p ce -> mkcl loc (Pcl_fun "" None (patt p) (class_expr ce))
+          (Pcl_fun (Optional lab) None (patt_of_lab loc lab p) (class_expr ce))
+    | CeFun loc p ce -> mkcl loc (Pcl_fun Nolabel None (patt p) (class_expr ce))
     | CeLet loc rf bi ce ->
         mkcl loc (Pcl_let (mkrf rf) (binding bi []) (class_expr ce))
     | CeStr loc po cfl ->
@@ -1424,7 +1426,7 @@ value varify_constructors var_names =
     | <:class_str_item< $cst1$; $cst2$ >> ->
         class_str_item cst1 (class_str_item cst2 l)
     | CrInh loc ov ce pb ->
-        let opb = if pb = "" then None else Some pb in
+        let opb = if pb = "" then None else Some (with_loc pb loc) in
         [mkcf loc (Pcf_inherit (override_flag loc ov) (class_expr ce) opb) :: l]
     | CrIni loc e -> [mkcf loc (Pcf_initializer (expr e)) :: l]
     | CrMth loc s ov pf e t ->
@@ -1448,7 +1450,7 @@ value varify_constructors var_names =
   value directive_arg =
     fun
     [ ExStr _ s -> Pdir_string s
-    | ExInt _ i -> Pdir_int (int_of_string i)
+    | ExInt _ i -> Pdir_int (i, None)
     | <:expr< True >> -> Pdir_bool True
     | <:expr< False >> -> Pdir_bool False
     | <:expr< >> -> Pdir_none
