@@ -163,6 +163,7 @@ New syntax:\
   Gram.Entry.clear opt_polyt;
   Gram.Entry.clear opt_private;
   Gram.Entry.clear opt_rec;
+  Gram.Entry.clear opt_nonrec;
   Gram.Entry.clear opt_virtual;
   Gram.Entry.clear opt_when_expr;
   Gram.Entry.clear patt;
@@ -209,14 +210,33 @@ New syntax:\
     else "-" ^ n
   ;
 
-  value mkumin _loc f arg =
+  value mkumin _loc arg =
     match arg with
     [ <:expr< $int:n$ >> -> <:expr< $int:neg_string n$ >>
     | <:expr< $int32:n$ >> -> <:expr< $int32:neg_string n$ >>
     | <:expr< $int64:n$ >> -> <:expr< $int64:neg_string n$ >>
     | <:expr< $nativeint:n$ >> -> <:expr< $nativeint:neg_string n$ >>
     | <:expr< $flo:n$ >> -> <:expr< $flo:neg_string n$ >>
-    | _ -> <:expr< $lid:"~" ^ f$ $arg$ >> ];
+    | _ -> <:expr< $lid:"~-"$ $arg$ >> ];
+
+  value mkumin_f _loc arg =
+    match arg with
+    [ <:expr< $flo:n$ >> -> <:expr< $flo:neg_string n$ >>
+    | _ -> <:expr< $lid:"~-."$ $arg$ >> ];
+
+  value mkuplus _loc arg =
+    match arg with
+    [ <:expr< $int:n$ >> -> <:expr< $int:n$ >>
+    | <:expr< $int32:n$ >> -> <:expr< $int32:n$ >>
+    | <:expr< $int64:n$ >> -> <:expr< $int64:n$ >>
+    | <:expr< $nativeint:n$ >> -> <:expr< $nativeint:n$ >>
+    | <:expr< $flo:n$ >> -> <:expr< $flo:n$ >>
+    | _ -> <:expr< $lid:"~+"$ $arg$ >> ];
+
+  value mkuplus_f _loc arg =
+    match arg with
+    [ <:expr< $flo:n$ >> -> <:expr< $flo:n$ >>
+    | _ -> <:expr< $lid:"~+."$ $arg$ >> ];
 
   value mklistexp _loc last =
     loop True where rec loop top =
@@ -516,8 +536,8 @@ New syntax:\
             Ast.StOpn _loc Ast.OvNil i
             (* <:str_item< open $i$ >> *)
 
-        | "type"; td = type_declaration ->
-            <:str_item< type $td$ >>
+        | "type"; rf = opt_nonrec; td = type_declaration ->
+            Ast.StTyp (_loc, rf, td)
         | value_let; r = opt_rec; bi = binding ->
             <:str_item< value $rec:r$ $bi$ >>
         | "class"; cd = class_declaration ->
@@ -604,8 +624,8 @@ New syntax:\
             Ast.SgOpn _loc Ast.OvOverride i
         | "open"; i = module_longident ->
             Ast.SgOpn _loc Ast.OvNil i
-        | "type"; t = type_declaration ->
-            <:sig_item< type $t$ >>
+        | "type"; rf = opt_nonrec; td = type_declaration ->
+            Ast.SgTyp (_loc, rf, td)
         | value_val; i = a_LIDENT; ":"; t = ctyp ->
             <:sig_item< value $i$ : $t$ >>
         | "class"; cd = class_description ->
@@ -710,8 +730,10 @@ New syntax:\
         | e1 = SELF; "lsr"; e2 = SELF -> <:expr< $e1$ lsr $e2$ >>
         | e1 = SELF; op = infixop4; e2 = SELF -> <:expr< $op$ $e1$ $e2$ >> ]
       | "unary minus" NONA
-        [ "-"; e = SELF -> mkumin _loc "-" e
-        | "-."; e = SELF -> mkumin _loc "-." e ]
+        [ "-"; e = SELF -> mkumin _loc e
+        | "-."; e = SELF -> mkumin_f _loc e
+        | "+"; e = SELF -> mkuplus _loc e
+        | "+."; e = SELF -> mkuplus_f _loc e ]
       | "apply" LEFTA
         [ e1 = SELF; e2 = SELF -> <:expr< $e1$ $e2$ >>
         | "assert"; e = SELF -> mkassert _loc e
@@ -1703,9 +1725,15 @@ New syntax:\
       ] ]
     ;
     opt_rec:
-      [ [ "rec" -> <:rec_flag< rec >>
+      [ [ "rec" -> Ast.ReRecursive
         | `ANTIQUOT ("rec"|"anti" as n) s -> Ast.ReAnt (mk_anti n s)
-        | -> <:rec_flag<>>
+        | -> Ast.ReNil
+      ] ]
+    ;
+    opt_nonrec:
+      [ [ "nonrec" -> Ast.ReNonrecursive
+        | `ANTIQUOT ("nonrec"|"anti" as n) s -> Ast.ReAnt (mk_anti n s)
+        | -> Ast.ReNil
       ] ]
     ;
     opt_override:
