@@ -69,7 +69,7 @@ value print_out_value ppf tree =
     | Oval_char c -> fprintf ppf "'%s'" (Char.escaped c)
     | Oval_string s ->
         try fprintf ppf "\"%s\"" (String.escaped s) with
-        [ Invalid_argument "String.create" -> fprintf ppf "<huge string>" ]
+        [ Invalid_argument _ -> fprintf ppf "<huge string>" ]
     | Oval_list tl ->
         fprintf ppf "@[<1>[%a]@]" (print_tree_list print_tree ";") tl
     | Oval_array tl ->
@@ -173,8 +173,7 @@ and print_simple_out_type ppf =
         [ Ovar_fields fields ->
             print_list print_row_field (fun ppf -> fprintf ppf "@;<1 -2>| ")
               ppf fields
-        | Ovar_name id tyl ->
-            fprintf ppf "@[%a%a@]" print_typargs tyl print_ident id ]
+        | Ovar_typ typ -> print_out_type_2 ppf typ ]
       in
       fprintf ppf "%s[%s@[<hv>@[<hv>%a@]%a ]@]" (if non_gen then "_" else "")
         (if closed then if tags = None then "= " else "< "
@@ -210,7 +209,8 @@ and print_simple_out_type ppf =
       }
   | Otyp_alias _ _ | Otyp_poly _ _ | Otyp_open
   | Otyp_arrow _ _ _ | Otyp_constr _ [_ :: _] as ty ->
-      fprintf ppf "@[<1>(%a)@]" print_out_type ty ]
+      fprintf ppf "@[<1>(%a)@]" print_out_type ty
+  | Otyp_attribute (_, _) -> ()]
   in
   print_tkind ppf
 and print_out_constr ppf (name, tyl, ret) =
@@ -368,7 +368,8 @@ and needs_semi =
   | Osig_type _ rs -> rs <> Orec_next
   | Osig_typext _ _
   | Osig_modtype _ _
-  | Osig_value _ _ _ -> True ]
+  | Osig_value _ -> True
+  | Osig_ellipsis -> False ]
 and print_out_signature ppf =
   fun
   [ [] -> ()
@@ -413,7 +414,10 @@ and print_out_sig_item ppf =
       print_out_type_decl
           (if rs = Orec_next then "and" else "type")
           ppf td
-  | Osig_value name ty prims ->
+  | Osig_value { oval_name  = name
+               ; oval_type  = ty
+               ; oval_prims = prims
+               ; _ } ->
       let kwd = if prims = [] then "value" else "external" in
       let pr_prims ppf =
         fun
@@ -425,7 +429,9 @@ and print_out_sig_item ppf =
             } ]
       in
       fprintf ppf "@[<2>%s %a :@ %a%a@]" kwd value_ident name
-        Toploop.print_out_type.val ty pr_prims prims ]
+        Toploop.print_out_type.val ty pr_prims prims
+  | Osig_ellipsis ->
+      fprintf ppf "..." ]
 
 and print_out_type_decl kwd ppf { otype_name    = name
                                 ; otype_params  = args
